@@ -7,16 +7,22 @@ from boto3.dynamodb.conditions import Attr, Key
 from .utils.dynamodb_utils import (batch_insert, get_item, get_item_and_retrieve_specific_attributes, put_item,
                                    query_by_key, query_by_key_min_max, scan_table, update_item)
 
-ALERTS_TABLE = 'alerts.items'
 ALERTS_CONFIGS_TABLE = 'alerts.config'
-METRICS_DETAILS_TABLE = 'metric.details'
-LOGS_METRICS_FETCHING_TABLE = 'logs.cloud_metric_fetching'
-LOGS_API_REQUEST_TABLE = 'logs.api_request'
-COMPONENTS_TABLE = 'component.info'
-ATOMIC_COUNTER_TABLE = 'atomic.counter'
+ALERTS_TABLE = 'alerts.items'
 ANOMALIES_TABLE = 'anomalies.items'
+ATOMIC_COUNTER_TABLE = 'atomic.counter'
+COMPONENTS_TABLE = 'component.info'
+METRICS_DETAILS_TABLE = 'metric.details'
+
+LOGS_API_REQUEST_TABLE = 'logs.api_request'
+LOGS_METRICS_FETCHING_TABLE = 'logs.cloud_metric_fetching'
+LOGS_ANOMALARM_ANOMALIES_WEBHOOK_TABLE = 'logs.anomalarm_anomalies_webhook'
+LOGS_ERRORS_TABLE = 'logs.errors'
 
 DEV_ANOMALIES_TABLE = 'dev.anomalies.items'
+DEV_LOGS_ANOMALARM_ANOMALIES_WEBHOOK_TABLE = 'dev.logs.anomalarm_anomalies_webhook'
+DEV_LOGS_API_REQUEST_TABLE = 'dev.logs.api_request'
+DEV_LOGS_ERRORS_TABLE = 'dev.logs.errors'
 
 
 def prepare_decimal(item):
@@ -112,8 +118,8 @@ def terminate_alert(alert_id, start_time, end_timestamp, dynamodb):
 # ########### METRICS ##########################
 # create new metric_timeseries item for current day
 # TODO add unit to create metric
-def create_metric(id, name, provider, namespace, region_name, agent, org_id, component_id, alignment, dimension_name,
-                  dimension_value, description, is_active, dynamodb):
+def create_metric(id, name, provider, namespace, region_name, data_center_id, agent, org_id, component_id, alignment,
+                  dimension_name, dimension_value, description, is_active, dynamodb):
   """
   create_metric(
     id = "test1", name = "error_rate",
@@ -129,6 +135,7 @@ def create_metric(id, name, provider, namespace, region_name, agent, org_id, com
     "provider": provider,
     "namespace": namespace,
     "region_name": region_name,
+    "data_center_id": data_center_id,
     "agent": agent,
     "org_id": org_id,
     "component_id": component_id,
@@ -181,7 +188,11 @@ def batch_insert_metric_details_objects(list_of_metric_objects, dynamodb):
 
 def get_metric_details(metric_id, dynamodb):
   key_dict = {"id": metric_id}
-  attr_list = ['id', 'name', 'namespace', 'alignment', 'dimension_name', 'dimension_value', 'org_id', 'is_active']
+
+  attr_list = [
+    'id', 'name', 'namespace', 'alignment', 'region_name', 'data_center_id', 'dimension_name', 'dimension_value',
+    'org_id', 'is_active'
+  ]
   metric_details = get_item_and_retrieve_specific_attributes(key_dict, attr_list, METRICS_DETAILS_TABLE, dynamodb)
   return metric_details
 
@@ -216,6 +227,13 @@ def batch_insert_component_info_objects(list_of_component_objects, dynamodb):
     batch_insert(component_list, COMPONENTS_TABLE, dynamodb)
 
 
+def update_component_last_fetched_ts(component_id, last_fetched_ts, dynamodb):
+  key_dict = {'id': decimal.Decimal(component_id)}
+  update_expression = 'set last_fetched_ts=:ts'
+  expression_attribute_values = {':ts': last_fetched_ts}
+  return update_item(key_dict, update_expression, expression_attribute_values, COMPONENTS_TABLE, dynamodb)
+
+
 # ############# ALERT CONFIGS ################
 def query_alerts_configs_by_key(metric_id, dynamodb):
   key_condition = Key('metric_id').eq(metric_id)
@@ -292,12 +310,43 @@ def insert_api_request_log(api_name, request_timestamp, response_status_code, re
   return response
 
 
+def insert_anomalarm_anomalies_webhook_log(anomalarm_id, timestamp, request, response, dynamodb, is_dev_env=False):
+  """
+  insert_anomalarm_anomalies_webhook_log(anomalarm_id='256828',
+                                         timestamp=1603466177,
+                                         request={...},
+                                         response={...},
+                                         dynamodb=dynamodb)
+  """
+  item = {
+    'anomalarm_id': anomalarm_id,
+    'timestamp': decimal.Decimal(timestamp),
+    'request': prepare_decimal(request),
+    'response': prepare_decimal(response)
+  }
+  table_name = LOGS_ANOMALARM_ANOMALIES_WEBHOOK_TABLE if not is_dev_env else DEV_LOGS_ANOMALARM_ANOMALIES_WEBHOOK_TABLE
+  return put_item(item, table_name, dynamodb)
+
+
+def insert_error_log(dynamodb, service_name, timestamp, msg, details, is_dev_env=False):
+  """
+  insert_error_log(dynamodb=dynamodb, service_name="metric_to_db", timestamp=1599563224, msg="Error inserting value",
+                   details={
+                     'exception': 'RejectedRecordsException',
+                     'response': {...}
+                   })
+  """
+  item = {'service_name': service_name, 'timestamp': decimal.Decimal(timestamp), 'error': msg, 'details': details}
+  table_name = LOGS_ERRORS_TABLE if not is_dev_env else DEV_LOGS_ERRORS_TABLE
+  return put_item(item, table_name, dynamodb)
+
+
 # ############# ANOMALIES ####################
 def insert_new_anomaly(id, timestamp, metric_id, value, dynamodb, is_dev_env=False):
   """
   insert_new_anomaly(id="125123", timestamp=1599563224, metric_id="m412", value=123.44, dynamodb=dynamodb)
   """
-  item = prepare_decimal({'id': id, 'timestamp': timestamp, 'metric_id': metric_id, 'value': value})
+  item = {'id': id, 'timestamp': decimal.Decimal(timestamp), 'metric_id': metric_id, 'value': decimal.Decimal(value)}
   table_name = ANOMALIES_TABLE if not is_dev_env else DEV_ANOMALIES_TABLE
   return put_item(item, table_name, dynamodb)
 
@@ -335,7 +384,7 @@ def update_anomaly_relations(id,
                            },
                            dynamodb=dynamodb)
   """
-  key_dict = prepare_decimal({'id': id, 'timestamp': timestamp})
+  key_dict = {'id': id, 'timestamp': decimal.Decimal(timestamp)}
   update_expression = 'set cross_correlations=:cc, possible_related_anomalies=:pra, possible_related_matches=:prm'
   expression_attribute_values = prepare_decimal({
     ':cc': cross_correlations,
@@ -350,9 +399,9 @@ def terminate_anomaly(id, timestamp, end_timestamp, dynamodb, is_dev_env=False):
   """
   terminate_anomaly(id="125123", timestamp=1599563224, end_timestamp=1599663224, dynamodb=dynamodb)
   """
-  key_dict = prepare_decimal({'id': id, 'timestamp': timestamp})
+  key_dict = {'id': id, 'timestamp': decimal.Decimal(timestamp)}
   update_expression = 'set end_timestamp=:ts'
-  expression_attribute_values = prepare_decimal({':ts': end_timestamp})
+  expression_attribute_values = {':ts': decimal.Decimal(end_timestamp)}
   table_name = ANOMALIES_TABLE if not is_dev_env else DEV_ANOMALIES_TABLE
   return update_item(key_dict, update_expression, expression_attribute_values, table_name, dynamodb)
 
