@@ -134,6 +134,14 @@ def update_item_conditionally(key_dict, condition_expression, update_expression,
     raise DynamoDBError(e.response['Error']['Message'])
 
 
+def add_attr_to_every_item_in_table(attr, attr_val, table, dynamodb):
+  '''
+  Function to update all items in a determined table, to add a new attribute
+  '''
+  # TODO: #49
+  return True
+
+
 def delete_item_conditionally(key_dict, condition_expression, expression_attr_values, table_name, dynamodb):
   '''
   condition_expression = "significance_score <= :val"
